#!/usr/bin/python
#! coding:utf-8 -*-

#from sys import path
from datetime import datetime
from string import split,strip
from MySQLdb import connect
from MySQLdb.cursors import DictCursor

from CGAInventorySystem.LOG import *
from CGAInventorySystem.DataBase import DataBase
from CGAInventorySystem.Exceptions import IncorrectTypeException

class Informe:
	''' Clase que da lugar al parseo del informe.xml enviado por un centro.
	Atributos:
	str 	codigo_centro
	long	version_regla
	[str,int] list  lista_exp_num
	xml_file string'''
	
	def __init__ (self, source=None, codigo_centro=None, version_regla=None, lista_exp_num=None):
		if isinstance(source, DataBase) and codigo_centro and version_regla:
			# cargar desde db
			LOG ( log.deb," Cargar desde DAtabase")
			self.init_from_dataBase( dataBase=source, codigo_centro = codigo_centro, version_regla = version_regla )
		elif isinstance( source, str):
				# cargar desde xml
				LOG( log.deb,  "Deserializando pickle para crear Informe")
				# comprobar si existe el archivo
				try:
					informe_tmp = open(source)
				except IOError:
					LOG (log.err, "El archivo no existe")
					return 1
				fd.close()
				self.version_regla = informe_tmp.version_regla
				self.codigo_centro = informe_tmp.codigo_centro
				self.lista_exp_num = informe_tmp.lista_exp_num
				self.create_time = informe_tmp.create_time
				LOG (log.deb, "Cargar desde el archivo %s" %str(source) )
		elif not source and codigo_centro and version_regla: 
			LOG (log.deb, "Instanciando informe desde argumentos: codigo_centro=%s version_regla=%s" %(codigo_centro, version_regla))
			#cargar desde parametros
			self.init_from_args(codigo_centro = codigo_centro, version_regla=version_regla, lista_exp_num=lista_exp_num)
		else:
			raise IncorrectTypeException ("Informe", "source", "__init__")

	
	def init_from_args( self, codigo_centro=None, version_regla=None, lista_exp_num=None):
		''' '''
		if not isinstance (codigo_centro, str):
			raise IncorrectTypeException ("Informe", "codigo_centro", "__init__")
		if not isinstance(version_regla, long):
			raise IncorrectTypeException ("Informe", "version_regla", " init_from_args")
		self.version_regla = version_regla
		self.codigo_centro = codigo_centro
		self.create_time = str(datetime.today())

		if lista_exp_num:
			for element in lista_exp_num:
				if len(element)!=2: 
					raise IncorrectTypeException ("Informe", "lista_exp_num", "Wrong len of lista_exp_num: %s" % str(len(element)))
				elif not isinstance(element[0],str) or not isinstance (element[1], int):
					raise IncorrectTypeException ("Informe", "lista_exp_num", "One tuple [exp, num] is invalid")
			else:
				self.lista_exp_num = lista_exp_num
		else:
			self.lista_exp_num = list()

			
	def init_from_dataBase( self, dataBase, codigo_centro, version_regla):
		''' Create the Informe object qurerying to dataBase'''
		mysql_select = 'SELECT tag, count(accountinfo.tag) AS count FROM accountinfo GROUP BY tag'
		data = dataBase.launch_query( mysql_select)
		lista = list()
		if data:
			for tuple in data:
				vector = ( tuple.get("tag"), int(tuple.get("count") ))
				lista.append( vector)
		self.init_from_args( codigo_centro, version_regla, lista)	


	def __str__( self):
		''' '''
		buffer =  "Codigo de centro="+str(self.codigo_centro)+"\n version_regla="+str(self.version_regla)
		for element in self.lista_exp_num:
			buffer=buffer+"\n  Expediente: "+str(element[0])+"	Cant: "+str(element[1])
		return buffer

	def set_version_regla (self, version_regla):
		''' '''
		if not isinstance(version_regla, long):
			raise IncorrectTypeException ("Informe", "version_regla", "set_version_regla")
		else:
			self.version_regla = version_regla

	def exp_handler( self, exp, num=0 ):
		'''The operation this function depends of the parameter num:
		    num == 0: removes the element of the list
				num >  0: change the amount for this exp	'''
		if not isinstance(exp,str):
			raise IncorrectTypeException ("Informe", "exp", "add_exp")
		if not isinstance(num,int):
			raise IncorrectTypeException ("Informe", "num", "add_exp")
		if num < 0:
			buffer = "Valor para %s menor que 0" %(exp)
			raise IncorrectTypeException("Informe","num",buffer)
		else:
			index = -1	
			for element in self.lista_exp_num:
				if element[0]==exp:
					index = self.lista_exp_num.index(element)
					break
			if num >0 and index!=-1:
				self.lista_exp_num[index]=[exp,num]
			elif num>0 and index==-1:
				self.lista_exp_num.append([exp,num])
			elif num==0 and index!=-1:
				self.lista_exp_num.remove(self.lista_exp_num[index])
			else:
				raise IncorrectTypeException("Informe", "num", "No existe el elemento a eliminar")

	def db_insert2(self, db_host, db_name, db_user, db_passwd):
			''' '''
			db_connect = connect( host=db_host,	user=db_user, passwd=db_passwd, db=db_name )
			cursor = db_connect.cursor(DictCursor)
			mysql_insert_informe = 'INSERT inv_informe (id_centro, version_regla, create_time, id_exp, cantidad) VALUES '
			for expediente in self.lista_exp_num:
				mysql_select_idexp = 'SELECT id_exp FROM inv_expediente WHERE cod_exp="%s"' %(expediente[0])
				cursor.execute(mysql_select_idexp)
				data = cursor.fetchone()
				if data:
					id_exp=data["id_exp"]
					LOG(log.deb, "Informe.db_insert. cod_exp="+expediente[0]+" ; id_exp="+str(id_exp))
					mysql_insert=mysql_insert_informe+'("%s", "%s", "%s" , "%s")'\
					%( self.codigo_centro, self.version_regla, id_exp, expediente[1])
					cursor.execute(mysql_insert)
				else:
					LOG(log.err, "Informe.db_insert. No existe id_exp para el expediente: %s" %(expediente[0]))
			cursor.close()
			db_connect.commit()
			db_connect.close()
			
	def db_insert(self, dataBase):
			''' '''
			mysql_insert_informe = 'INSERT inv_informe (id_centro, version_regla, create_time, id_exp, cantidad) VALUES '
			for expediente in self.lista_exp_num:
				mysql_select_idexp = 'SELECT id_exp FROM inv_expediente WHERE cod_exp="%s"' %(expediente[0])
				data = dataBase.launch_query( mysql_select_idexp)
				if data:
					tupla = data[0]
					id_exp = tupla["id_exp"]
					LOG(log.deb, "Informe.db_insert. cod_exp="+expediente[0]+" ; id_exp="+str(id_exp))
					mysql_insert=mysql_insert_informe+'("%s", "%s", "%s", "%s" , "%s")'\
					%( self.codigo_centro, self.version_regla, self.create_time, id_exp, expediente[1])
					id = dataBase.launch_insert( mysql_insert)
					LOG( log.deb, "Introducido instancia de informe con id=%s" %id)
				else:
					LOG(log.err, "Informe.db_insert. No existe id_exp para el expediente: %s" %(expediente[0]))


	def get_id_exp (self, db_host, db_name, db_user, db_passwd, lista_exp_num ):
		''' Devuelve el id_exp a partir de la cadena que representa al expediente'''
		new_list = list()
		db_connect = connect (host=db_host,	user=db_user, passwd=db_passwd, db=db_name)
		cursor = db_connect.cursor(DictCursor)
		for element in lista_exp_num:
			mysql_query = 'SELECT id_exp FROM inv_expediente WHERE cod_exp="'+str(element[0])+'";'
			cursor.execute(mysql_query)
			data = cursor.fetchall()
			for row in data:
				new_list.append( [str(row["id_exp"]),element[1]] )
		cursor.close()
		db_connect.commit()
		db_connect.close()
		return new_list
	
	def already_exists( self, dataBase):
		''' Return true if in the database already exists a report
		for the same Centro'''
		count_query = 'SELECT COUNT(*) as count FROM inv_informe WHERE id_centro="%s" AND version_regla="%s" AND create_time="%s";' %( str(self.codigo_centro), str(self.version_regla), self.create_time)
		data = dataBase.launch_query( count_query)
		row1=data[0]
		resultado = row1.get("count")
		if resultado == 0:
			existe = False
		else:
			existe = True 
		return existe
