#!/usr/bin/python
# -*- coding: utf-8 -*-

from string import strip
from MySQLdb import connect
from MySQLdb.cursors import DictCursor
from time import localtime
from pickle import dump, PickleError

# modulos propios
from CGAInventorySystem.Exceptions import IncorrectTypeException, DuplicateDictionaryKeyException
from CGAInventorySystem.RulesClass.Rules import Rules
from CGAInventorySystem.RulesClass.Query import Query
from CGAInventorySystem.RulesClass.Key_tag import Key_tag
from CGAInventorySystem.DataBase import DataBase, OcsDataBase
from CGAInventorySystem.DbWorker import DbWorker
from CGAInventorySystem.LOG import *

	

class Rules_dictionary:
	''' CLASE Rules_dictionary 
	implementa un diccionario de tipo
	{ etiqueta : lista de consultas }
	Se usa al parsear el xml '''

	def __init__(self, source=None, version=None, dictionary={}, tag=None):
		''' '''
		if isinstance(source, DataBase):
			# cargar desde database OJO CON VERSION
			self.init_from_db( dataBase=source, version=version, tag=tag)
		elif not source and version and dictionary:	
			LOG( log.deb, "Cargar desde parámetros")
		else:
			raise IncorrectTypeException("Informe", "source", "Wrong attribute in __init__")
			

	def init_from_args( self, version="", dictionary={}):
		''' '''
		if not isinstance(dictionary, dict):
			raise IncorrectTypeException\
			("Rules_dictionary", "dictionary", " Wrong attribute type in init_from_args")
		if not self.check_keys_type( dictionary):
			raise IncorrectTypeException\
			("Rules_dictionary", "dictionary", " Wrong key(s) type in init_from_args")
		if not self.check_values_type( dictionary):
			raise IncorrectTypeException\
			("Rules_dictionary", "dictionary", " Wrong value(s) type in init_from_args")
		if not isinstance(version, str):
			raise IncorrectTypeException\
			("Rules_dictionary", "version","Invalid object version atribute in init_from_args")
		self.version = version
		self.dictionary = dictionary

	
	def init_from_db( self, dataBase, version=None, tag=None):
		''' '''
		if not isinstance( dataBase, DataBase):
		# el objeto  no es DataBase
			raise IncorrectTypeException\
			("Rules_dictionary", "dataBase","Invalid object dataBase atribute in init_from_db")
		if version and not isinstance( version, long):
			# la version no es long 
			raise IncorrectTypeException\
			("Rules_dictionary", "version","Invalid object version atribute in init_from_db")
		#FIN COMPROBACION DE ERRORES

		# comprobamos el tipo de dataBase	
		if isinstance( dataBase, DataBase) and not isinstance( dataBase, OcsDataBase):
			# cargar desde base de datos `inventariado`
			if not version:
				# buscaremos los datos de la última regla introducida
				version = self.get_last_version( dataBase)
			elif not self.existe_version( dataBase, version):
				# la version no existe
				LOG( log.err, "No existe ninguna regla con version %s en la base de de datos %s" %(str(version), dataBase.db_name))
				return 1
			# cargar con datos de la version pasada como parametro
			LOG (log.deb, "El objeto sera creado en init_from_version")
			rules_object = self.init_from_version(version=version,dataBase= dataBase)
		elif isinstance( dataBase, OcsDataBase):
			if not tag and not isinstance(tag, str):
			# instanciamos el objeto con todos las reglas en
			# dataBase de tipo `ocsweb`
				self.init_from_hw_id( dataBase)

	def get_last_version(self, dataBase):
		''' Return a long with the last Regla version 
		inserted in the dataBase'''
		sql_query = 'SELECT DISTINCT(regla.version) as version FROM regla ORDER BY VERSION DESC LIMIT 1'
		data = dataBase.launch_query( query=sql_query)
		tupla = data[0]
		version = long( tupla["version"] )
		return version


	def init_from_hw_id( self, dataBase):
		''' Instancia el objeto con los valores obtenidos en la base de datos dataBase
		dataBase ha de ser de tipo OcsDataBase'''
		q1 = Query("hardware") 
	 	q1.add_field_value("PROCESSORT","delete")
	 	q2 = Query("bios")
	 	q2.add_field_value("BMANUFACTURER","to_delete")
		q2.add_field_value("SMANUFACTURER","to_delete")
	 	q2.add_field_value("SMODEL","to_delete")
	 	q3 = Query("controllers")
		q3.add_field_value("MANUFACTURER","to_delete")
 		querys_list_tmp = [q1,q2,q3]

		self.version = self.get_current_time()
		self.dictionary = {}
		hw_list = self.get_hw_id_list(dataBase)
		for hw_id_element in hw_list:
				
			# con esto generamos una lista de querys para el reporte con hw_id
			rules_querys = self.fill_querys_values( querys_list=querys_list_tmp, hw_id=hw_id_element, dataBase=dataBase)
			# este rules_querys trae Querys con campo value vacio. Los limpiamos
	#		new_rules_querys = self.clear_empty_querys(rules_querys)
			rules_ob = Rules(querys=rules_querys,comment="Portatil ACME2010")
	#		rules_ob.remove_empty_querys()
			# obtenemos el tag del hw_id
			hw_tag = self.get_tag( dataBase=dataBase, hw_id = hw_id_element)
			key_tag_ob = Key_tag( tag=hw_tag)
			self.add_key_value( key_tag_ob, rules_ob)
	
	def get_hw_id_list( self, dataBase):
		''' Devuelve una lista con todos los id hardware.
		La base de datos sera de tipo ocsweb'''
		if not isinstance(dataBase, OcsDataBase):
			# el parametro no es una base de datos tipo ocsweb
			raise IncorrectTypeException("Rules_dictionary","dataBase", "attribute DataBase given to function get_hw_id_list is not a OcsDataBase object")
		mysql_select = 'SELECT DISTINCT(id) as id FROM hardware;'
		data = dataBase.launch_query( mysql_select)
		hw_id_list = list()
		for element in data:
			hw_id_list.append(element['id'])
		return hw_id_list	


	def get_current_time( self):
		''' Return at a str the current time: YYYYMMDDHHMMSS'''
		localTime = localtime()
		current_time = ( localTime.tm_year, localTime.tm_mon, localTime.tm_mday, localTime.tm_hour, localTime.tm_min, localTime.tm_sec)
		final_time = str()
		for element in current_time:
			#ind = current_time.index(element)
			str_element = str(element)
			if len(str_element) == 1:
				str_element = "0"+str_element
			final_time = final_time+str_element
		return final_time

	def get_tag ( self, dataBase, hw_id):
		''' Devuelve el tag asociado al hardware_id pasado como parametro'''
		mysql_tag_query = 'SELECT accountinfo.hardware_id AS id, accountinfo.tag AS tag FROM accountinfo WHERE accountinfo.hardware_id = "%s";' %( str(hw_id))
		data = dataBase.launch_query( query=mysql_tag_query)
		if not data:
			hw_id = None
		else:
			# siempre devolvera el id
			# de la primera tupla
			row = data[0]
			tag = row["tag"]
		return str(tag)
	
	
	def fill_querys_values (self, querys_list, hw_id, dataBase):
		''' Return a list [] of Querys. 
		It will be used to create de xml file '''
		final_list = [] 
		for query in querys_list:
			table = query.get_table()
			final_query = Query(table=table)
			fields_list = query.values_dict.keys()
			for field_element in fields_list:
				#if the table is hardware, joins are not necesary 
				if not table == "hardware":
						query_from="FROM hardware INNER JOIN "
						query_from = "%s %s ON hardware.id=%s.hardware_id" %(query_from,table,table)
				else:
					query_from = "FROM hardware"
				query_select="SELECT DISTINCT hardware.id AS hw_id, "
				query_where="WHERE hardware.id='%s'" %(hw_id)
				query_select = "%s %s.%s AS value " %(query_select, table, field_element)
				query_string = "%s\n%s\n%s" %(query_select, query_from, query_where)
				if table == "controllers":
					extra_ethernet = "AND controllers.name  LIKE '%Ethernet%'"
					extra_vga =" AND controllers.name LIKE '%vga%'"
					extra_audio =" AND controllers.name LIKE '%audio%'"
					list = [extra_ethernet, extra_vga, extra_audio]
					for elto_extra in list:
						controller_query= "%s %s\n" %(query_string, elto_extra)
						data = dataBase.launch_query( query=controller_query)
						for element in data:
							info_requed = element["value"]
							final_query.add_field_value(field_element, info_requed)
				else:
					data = dataBase.launch_query( query=query_string)
					result = data[0]
					info_requed  = result["value"]
					# the info obtained is added to the object
					final_query.add_field_value(field_element, info_requed)
			final_list.append(final_query)
		return final_list


	def serialize(self, file_name=None, folder=None):
		'''Serialize self object. use file_name parameter for file's name
		Default attributes values:
		  file_name = ruleYYYMMDDHHMMSS.pick  datetime is self.version
			folder  = /usr/var/CGAInventorySystem/reglas/'''
		if not file_name:
			file_name = folder+"rule"+str(self.version)+".pick"
		else:
			file_name = folder+file_name
		try:
			fd = open( file_name, "w")
		except IOError:
			LOG( log.err, "No se puede crear el archivo %s" %(file_name))
		dump(self, fd)
		LOG( log.deb, "Creado con pickle la regla: %s" %(folder+file_name))
		fd.close()
		return file_name

	def existe_tag (self, dataBase, tag):
		''' Return True if tag is inserted in DataBase'''
		mysql_query = 'SELECT COUNT(accountinfo.tag) as tag FROM accountinfo WHERE accountinfo.tag="%s"' %(str(version))
		data = dataBase.launch_query( query=mysql_query)
		tupla = data[0] 
		if tupla["count"] > 0:
			existe = True
		else:
			existe = False
		return existe


	def init_from_version( self, dataBase ,version=None):
		''' Inicializate the own values of the attributes
		<<version>> and dictionary. It is done querying and getting 
		values from <<dataBase>>'''
		self.version = str(version)
		self.dictionary = {}
		# Creamos la consulta para recorrer los expedientes
		# que correspondan con la version de regla <<version>>
		mysql_select_expedientes = 'SELECT DISTINCT(expediente.cod_exp) AS exp from regla join expediente on regla.id_exp = expediente.id_exp WHERE regla.version="'+str(version)+'";'
		data_expedientes = dataBase.launch_query( query=mysql_select_expedientes)
		if not data_expedientes:
			return 1
		for expediente_tuple in data_expedientes:
			exp = expediente_tuple["exp"]
			rules = Rules()
			tag_o = Key_tag(exp)
			mysql_select_tabla = 'SELECT distinct( inv_hardware.tabla) as tabla FROM inv_expediente JOIN inv_regla ON inv_regla.id_exp = inv_expediente.id_exp JOIN inv_hardware ON inv_regla.id_hardware = inv_hardware.id_hardware WHERE inv_regla.version="%s" AND inv_expediente.cod_exp="%s"' %( self.version, exp)
			data_tablas = dataBase.launch_query( query=mysql_select_tabla)
			for tuple_hardware_tabla in data_tablas:
				tabla = tuple_hardware_tabla["tabla"]
				query = Query(table=tabla)
				mysql_select_campo_valor = 'SELECT inv_hardware.campo as campo, inv_hardware.valor as valor FROM inv_expediente JOIN inv_regla ON inv_regla.id_exp = inv_expediente.id_exp JOIN inv_hardware ON inv_regla.id_hardware = inv_hardware.id_hardware WHERE inv_regla.version="%s" AND inv_expediente.cod_exp="%s" AND inv_hardware.tabla="%s"' %( self.version, exp, tabla)
				data_campo_valor = dataBase.launch_query( query=mysql_select_campo_valor)
				for tuple_hardware_campo_valor in data_campo_valor:
					campo = tuple_hardware_campo_valor["campo"]
					valor = tuple_hardware_campo_valor["valor"]
					query.add_field_value(new_field=campo , new_value=valor)
				rules.add_query(query)
			self.add_key_value( tag_o, rules)


	def existe_version (self, dataBase, version):
		''' Return True if version is inserted in DataBase'''
		mysql_query = 'SELECT COUNT( inv_regla.version) as count FROM inv_regla WHERE inv_regla.version="%s"' %(str(version))
		data = dataBase.launch_query( query=mysql_query)
		tupla = data[0] 
		if tupla["count"] > 0:
			existe = True
		else:
			existe = False
		return existe
	

	def __str__(self):
		''' '''
		if len(self.dictionary) == 0:
			_buffer = "Rules_dictionary is empty\nVersion %s" %(self.vesrion)
		else:
			_buffer = "Dictionary version %s \nDICTIONARY ELEMENTS:\n" %(self.version)
			_keys = self.dictionary.keys()
			for _key_element in _keys:
				_buffer = _buffer + str(_key_element) + str(self.dictionary[_key_element])
		return _buffer

	
	def check_keys_type (self, dictionary):
		''' Function to check key's type of dictionary (Key_tag ) '''
		if not isinstance(dictionary,dict):
			raise IncorrectTypeException\
			("Rules_dictionary","dictionary","Wrong attribute type in checkKeysTypes ")
		else:
			keys = dictionary.keys()
			for element in keys:
				if not isinstance(element,Key_tag):
					return False
			return True
		
	
	def check_values_type (self, dictionary):
		''' Function to check values's type of dictionary ( Rules ) '''
		if not isinstance(dictionary,dict):
			raise IncorrectTypeException\
			("Rules_dictionary","dictionary","Wrong attribute type in checkValuesTypes ")
		else:
			values = dictionary.values()
			for element in values:
				if not isinstance(element,Rules):
					return False
			return True
	
	def add_key_value(self, key, rules):
		''' Add new keys to self Rules_dictionary '''
		if not isinstance(key,Key_tag):
			raise IncorrectTypeException\
         ("Rules_dictionary","key","Wrong attribute type in add_key_value ")
		elif not isinstance(rules,Rules):
			raise IncorrectTypeException\
         ("Rules_dictionary","value","Wrong attribute type in checkValuesTypes ")
		else:
			if key in self.dictionary:
				raise DuplicateDictionaryKeyException ( "Rules_dictionary" )
			else:
				self.dictionary[key]=rules


	def is_inserted ( self, database):
		'''Checks if it's inserted self Rules_dictionary in <<database>>'''
		if not isinstance( database, DataBase):
			 raise IncorrectTypeException( "Rules_dictionary", "database", "function inserted" )
		mysql_query = 'SELECT EXISTS ( SELECT inv_regla.id_regla from inv_regla WHERE version ="'+self.version+'") as existe'
		data = database.launch_query( query=mysql_query)
		existe = data[0]
		if existe["existe"] == 1 :
			insertado = True 
		else:
			insertado = False 
		return insertado

	
	def get_idexp_hwid_list( self,dataBase, cod_exp, rule):
		'''Este método comprueba si existe el expediente pasado como argumento <<exp>>
		En caso negativo crea uno nuevo y recorre el objeto de reglas <<rule>> 
	  comprobando si existe una restricción en la base de datos y creandolas en 
		caso de no existir..
		Devuelve: ( id_exp2, ( id_hw1, id_hw2, ...) '''

		if not isinstance(rule, Rules):
			raise IncorrectTypeException("Rules_dictionary", "rule", "get_idexp_hwid_list")
		if not isinstance( dataBase, DataBase):
			raise IncorrectTypeException("Rules_dictionary","dataBase", "get_idexp_hwid_list")
		# FIN COMPROBACION PARAMETROS
		lista = []
		id_exp = int(self.get_id_exp( dataBase, cod_exp))
		for query_element in rule.querys:
			list_tmp = self.get_id_hardware( dataBase=dataBase, query=query_element)
			for element in list_tmp: 
				lista.append(element)
		return ( id_exp, lista)


	def get_id_hardware( self, dataBase, query):
		'''Consulta en la base de datos y devuelves 
		una lista con los id`s correspondiente a la query'''
		id_lista = [] 
		tabla = query.table
		for field in query.values_dict.keys():
			values_list = query.values_dict.get(field)
			for value in values_list:
				mysql_query = 'SELECT inv_hardware.id_hardware from inv_hardware where tabla="%s" and campo="%s" and valor="%s"' %(tabla, field, value)
				data = dataBase.launch_query( query=mysql_query)
				if data:
					hw_id = data[0]["id_hardware"]
				#	LOG( log.deb, "Existe  Query: %s" %(str(hw_id)))
				else:
					insert_hardware = 'INSERT inv_hardware ( tabla, campo, valor) VALUES ( "%s", "%s", "%s")' %(tabla, field, value)
					LOG( log.deb, "Insertar hardware: %s" %insert_hardware)
					hw_id = dataBase.launch_insert( insert_hardware)
				#	LOG( log.deb, "Insertada query con id: %s" %( str( hw_id)))
				id_lista.append(int( hw_id))
		return id_lista
		
	

	def get_id_exp ( self, dataBase, exp):
		''' Consulta el id_exp a partir del codigo_expediente
		Si no existe lo inserta. Devuelve id_exp'''
		mysql_query = 'SELECT COUNT(*) AS count FROM inv_expediente WHERE inv_expediente.cod_exp="%s"' %(exp)
		data = dataBase.launch_query( mysql_query)
		tuple = data[0]
		if tuple["count"]==1:
			mysql_select = 'SELECT inv_expediente.id_exp FROM inv_expediente WHERE inv_expediente.cod_exp="%s"' %( exp)
			data1 = dataBase.launch_query(mysql_select)
			tuple1 = data1[0]
			id_exp = tuple1["id_exp"]
		if tuple["count"] == 0:
			#ya está registrado el expediente
			insert_exp = 'INSERT inv_expediente (cod_exp) VALUES ("%s")' %(str(exp))
			id_exp = dataBase.launch_insert( insert_exp)
		return id_exp



	def db_insert(self, dataBase):
		'''Método propio de objetos Informe para insertarlos 
		después de haber sido parseados.
		Se insertarán en la base de datos con nombre <<db_name>> 
		alojada en el equipo <<db_host>> mediante el usuario <<db_user>>
		con contraseña <<db_passwd>>'''
		# comprobacion de parametros
		if not isinstance( dataBase, DataBase):
			# objeto dataBase incorrecto
			LOG (log.err, "La base de datos donde insertar la regla es incorrececta")
			raise IncorrectTypeException 	("database", "Rules_dictionary")
		if not hasattr(self, "version"):
			# no existe el atributo version
			LOG(log.err, "Se hace db_insert de un objeto Rules_dictionary con atributo <<version>> sin inicializar" )
			return (0)
		if not hasattr(self, "dictionary"):
			LOG(log.err, "Se hace db_insert de un objeto Rules_dictionary con atributo <<dictionary>> sin inicializar" )
			return (0)
		#FIN COMPROBACIONES

		#comprobando si el objeto ya está insertado
		if self.is_inserted( dataBase):
			LOG( log.err, "La regla con version %s ya se encuentra en la base de datos %s"	%(self.version, dataBase.db_name))
			return 0
		else:
			dict = {}
			LOG( log.deb, "Insertando regla con version %s" %str(self.version))
			for key_tag_element in self.dictionary.keys():
				rule_object = self.dictionary.get( key_tag_element)
				# obtenemos ([id_exp,(hwid1, hwid2...)])
				list_idexp_idhw = self.get_idexp_hwid_list( dataBase = dataBase, cod_exp= key_tag_element.tag, rule=rule_object)
				dict[list_idexp_idhw[0]]=list_idexp_idhw[1]
				mysql_insert1 = 'INSERT inv_regla(version, id_exp, id_hardware) VALUES '
			for id_exp in dict.keys():
				list_hw = dict.get(id_exp)
				mysql_insert_tmp=""
				for hw_id in list_hw:
					mysql_insert_tmp = mysql_insert_tmp+' ("%s", "%s", "%s"),' %( self.version, str(id_exp), str( hw_id))
				mysql_insert = mysql_insert1 + mysql_insert_tmp[:-1]
				LOG( log.deb, "Insertado expediente id=%s" %id_exp)
				dataBase.launch_query (mysql_insert)
